# boke

Static blog generator. (尽可能简单的静态博客生成器)

- <https://docs.getpelican.com/en/latest/install.html> (参考)
- <https://github.com/pallets/markupsafe/>
- <https://doc.qt.io/qtforpython/index.html>
- <https://github.com/executablebooks/markdown-it-py>
- <https://github.com/lepture/mistune> (选定)
- <https://github.com/miyuchina/mistletoe/blob/master/performance.md> (已知问题)
- <https://support.typora.io/zh/Markdown-Reference/> (Markdown 简明教程)
- <https://css-tricks.com/no-class-css-frameworks/>

## 特色

命令行与 GUI 的诡异结合：既具有命令行的简洁、直接，又能利用 GUI 来弥补命令行的缺点（比如不够直观、字符转义问题）。

- 大量使用 GUI, 但每个 GUI 窗口都需要用命令行调出。
- GUI 采用 PyQt, 而且不是一个大而全的、完整的 GUI 软件，而是每次只弹出一个简单的对话框，因此消耗资源极少，感觉很轻，“啪”一下子就弹出来了，没有 Electron 软件的笨重感。


## init (初始化)

如下图所示，初始化一个博客简直不能更简单了，只要填写博客名称、作者名称即可，有 GUI 界面，非常直观。

![boke-init.jpg](https://vip2.loli.io/2022/04/09/NsGbSw56uUFQg7z.jpg)

详细说明如下：

```txt
$ mkdir my-blog (新建一个空文件夹)
$ cd my-blog (进入空文件夹内)
$ boke init (弹出 GUI 窗口填写博客名称、默认作者，如果检测到当前文件夹不是空的，会拒绝初始化)
$ ls (成功初始化后，可以看到以下文件与文件夹)

drafts (草稿：待发布、编辑中的文章请放在这里)
posted (已发布文章会被移动到该文件夹内)
output (把 posted 文件夹里的 Markdown 文件转成 HTML 文件输出到该文件夹)
templates (Jinja2模板 与 CSS文件)
boke.db (数据库)
```

### 更改博客资料

可使用如下命令，在弹出的 GUI 窗口中修改博客名称、默认作者等信息。

```sh
boke --blog-info
```


## 写文章

如前文所述，初始化后得到一些文件夹，请在 drafts 文件夹里新建文件写文章，文件后缀建议用 ".md", 文件内容必须采用 Markdown 格式, 必须采用 utf-8 编码。市面上有很多支持 Markdown 的文本编辑器，比如 [Typora](https://typora.io/) 和 [VS Code](https://code.visualstudio.com/), 可以找一个喜欢的来用。

- 参考: [Markdown 简明教程](https://support.typora.io/zh/Markdown-Reference/)
- 但要注意，本软件采用 [mistune](https://github.com/lepture/mistune) 做从 Markdown 到 HTML 的转换, mistune 注重速度，转换速度极快，但支持的特性、功能比较少，只支持上面 _Markdown 简明教程_ 中的一部分特性。


## Post (发布文章)

```sh
boke post ./drafts/filename.md
```

该命令从 drafts 文件夹中选择一个文件进行发布，会弹出一个 GUI 窗口，方便选择类别及填写作者、日期、标签、文件名等信息。

虽然看起来有很多项目，但其中多数项目已自动填写内容，必须操作的只有一项 Category(文章类别), 选择类别后直接按回车键，就可以完成发布操作。

![boke-post-r9y42o.jpg](https://vip2.loli.io/2022/04/07/2YHnBCbejZUKTys.jpg)

### ID

每篇文章会自动分配一个 ID, 一旦发布成功，文件名由该 ID 加后缀 ".md" 组成。

用户可更改文章 ID, 但只能使用 0-9, a-z, A-Z, _(下划线), -(短横线)。

### File (待发布文件)

是指即将发布的文件，不可在 GUI 界面更改文件，如果想发布另一个文件，请点击 Cancel 按钮，重新执行 'boke post' 命令指定要发布的文件。

### Title (标题)

自动获取文章第一行作为标题，不可在 GUI 界面修改标题。如果想修改标题，请点击 Cancel 按钮，修改文章里的第一句话，然后再重新执行 'boke post' 命令。

### Author (作者)

已自动填写默认作者，可以直接修改。每篇文章可以独立更改作者名。

另外，可使用命令 'boke --set-author' 更改默认作者。

### Category (文章类别)

比如 “技术”、“生活”、“书影音” 等，每篇文章必须选择一个类别。

### Datetime (发布日期)

日期自动填写当前时间，采用 RFC3339 格式，可直接修改。

### Tags (标签)

每篇文章可以有零个或多个标签，用井号(#)、逗号或空格分隔，不可包含 !@$%^&* 之类的特殊字符。填写标签后可点击 preview 按钮检查标签是否正确。

### 提示与快捷键

在 'boke post' 的 GUI 窗口，鼠标停留在文本框上会显示提示说明。

按住 ALT 键同时按 i 或 a, d, t, p 可分别激活 ID, Author, Datetime, Tags, preview。按 ESC 键可关闭窗口（相当于点击 Cancel 按钮）。


## Generate (生成 HTML 与 RSS)

根据 posted 里的 Markdown 文件与 templates 里的 Jinja2 模板，生成 HTML 与 RSS, 并复制 CSS, LICENSE 等资源文件到 output 文件夹。

- `boke gen -theme simple` (只生成新文章与有更新的文章)
- `boke gen -theme water -all/--force-all` (重新生成全部文章)
- `boke gen -theme mvp --ignore-assets` (不复制 CSS, LICENSE 等文件)
- `boke gen -theme keep` (保持当前主题不变)

- 如果 CSS, LICENSE 等资源文件没有变化（大多数情况下都没变化），可使用 '--ignore-assets' 避免无用的复制。如果不添加 '--ignore-assets' 参数，则会把 templates 文件夹里的全部资源文件都复制到 output。
- boke 的一切命令都不会删除 output 里的任何文件，只会添加或覆盖。因此，如果有多余的文件需要用户自己手动删除。
- 添加新文章与更改博客资料后执行 'boke gen' 会更新 RSS, 其他操作（比如更改文章内容、更改标签等）不会更新 RSS。

### Themes (CSS主题)

第一次使用命令 'boke gen' 生成静态文件时，必须选择主题，例如：

- `boke gen -theme simple` (选择 [simple.css](https://simplecss.org/) 主题)
- `boke gen -theme water` (选择 [water.css](https://watercss.kognise.dev/) 主题，顺便提一句，作者是一位美女程序员)
- `boke gen -theme mvp` (选择 [mvp.css](https://andybrewer.github.io/mvp/) 主题)

从第二次使用 'boke gen' 命令开始，可以不指定主题，表示保持当前主题不变。而且，还可以用 '--ignore-assets' 避免复制资源文件（比如 LICENSE.txt 等不经常改变内容的文件）。

```sh
boke gen --ignore-assets
```

### 自定义主题（一个例子）

除了自带的几个主题之外，用户还可以对 templates 里的模板进行修改，采用 [Jinja2](https://jinja.palletsprojects.com/en/3.1.x/templates/) 语法，也可以自己添加 CSS、JS。

项目源码里有一个例子 <https://github.com/ahui2016/boke/tree/main/themes/sakura>
在该例子中，对模板 base.html 进行了一点点修改，并且增加了 sakura.css 等两个 css 文件，同时也把许可证 CC0-1.0 改成 All rights reserved, 删除了 LICENSE.txt。

使用这个 sakura 主题的方法是：

(下面 **templates** 是指使用 'boke init' 命令生成的 templates 文件夹)

1. 把 templates 里的全部文件移动到其他地方
2. 下载 [sakura](https://github.com/ahui2016/boke/tree/main/themes/sakura) 里的全部文件，放进 templates 里
3. 执行命令 `boke gen -all`

完成。 output 文件夹里的文件已采用 sakura 主题重新生成。

生成几种目录：

- 最近更新
- 类别
- 标签
- 日期（年月）

每个类别与标签下只列出 n 篇文章，加一个 more 按钮。


## Category (类别)

每篇文章都必须属于一个类别，每篇文章的 prev 和 next 按钮是在指类别内的上一篇、下一篇，每个类别可以有类别说明（类似于“卷首语”之类的东西）。

- `boke cat -l/--list` (列出全部类别名称)


## Update (更新文章)

- 使用命令 `boke update ./posted/article-id.md`, 会弹出 GUI 窗口，可以更新文章的 ID, 文件名、标题、 类别、日期、标签等等。
- 文章的 ID 与它的文件名、网址是一一对应的，也就是说如果修改了一篇文章的 ID, 它的文件名、网址也会随之改变。
- 如果只修改文章的内容，不修改类别、标签等，可以使用 '--date-only' 参数，更快捷方便（不会弹出 GUI 窗口）。例如 `boke update --date-only ./posted/article-id.md`。

### 修改文章内容的一般流程

1. 大多数情况下，对于用 `boke post` 命令发布后的文章，可以直接进 posted 文件夹里用 Typora 之类的编辑器打开文件，直接修改。
2. 修改文件内容后，要记得执行 `boke update --date-only ./posted/article-id.md`, 该命令会更新数据库里这篇文章的 updated 时间。
3. 执行 `boke gen -ia` 命令重新生成静态文件，该命令会自动筛选 'boke update' 命令更新过日期的文章，忽略未更新的文章，从而提高效率。另外可以使用命令 `boke gen -all` 强制重新生成全部文章。


## Search (查找)

查找功能交给 [ripgrep](https://github.com/BurntSushi/ripgrep) 去做！


## LICENSE (许可证)

使用本软件生成的博客，默认声明了 CC0-1.0 许可证，意味着允许他人免费转载你的文章（包括商用）。如果不想采用 CC0 许可证，可以在 'boke init' 之后进入 templates 文件夹，删除 LICENSE.txt 文件，并删除 article.html 底部的这句话 `<footer>LICENSE <a href="../LICENSE.txt">CC0-1.0</a></footer>`

另外，如果采用其他许可证，可以替换 LICENSE.txt 文件的内容，并修改 footer 的许可证名称。

参考: <https://github.com/ahui2016/boke/tree/main/themes/sakura>


## 已知缺点

- 不保存图片，需要另外配合图床使用。
- 由于配合 GUI 窗口使用，很多功能不能通过纯命令行操作，因此无法自动化操作。
